/****************************************************
   TORNADO-PX64xxQ Demo Utility.
   DSP-to-DSP communication demo. rev.1A
   (C) MicroLAB Systems, 2003


   Description:
   ------------
	This demo tests on-board DSP-to-DSP communication. No host DSP activity is involved
	into this demo.

		General notes:
			1.	This demo must be loaded and started at all DSP nodes. All four DSP nodes
				are involved into this demo.
			2.	In order to synchronously start demos at all DSPs, the following is correct
				procedure for demo initialization:
					- load demo to all DSP environments
					- start DSP-B, DSP-C, DSP-D in any sequence. After initialization, they
					  will wait for synchronous signal over corresponding IRQ from the DSP-A
					- start DSP-A as the last of all DSPs, since DSP-A generates synchro-IRQs
					  to all other DSPs in order to start demo synchronously
				order to run successfully. All four DSP nodes are involved into this demo.
			3.	The following test procedures are included into this demo:
            		- DSP-to-DSP communication via horizontal/vertical/diagonal serial links
            		- DSP-to-DSP communication via horizontal/vertical/diagonal interrupt
					  requests
					- DSP-to-DSP communication via SSRAM with utilization of DSP node
					  interrupt request via corresponding SSRAM memory locations (if SSRAM
					  is installed)
            4.	Vertical and diagonal links shall be enabled via HOST LINK CONFIGURATION
            	register in order to perform complete test. In case particula link (diagonal
            	or vertical or both) is/are disabled, then the corresponing tesp procedure will
            	be skipped.
			5.	Each DSP uses its node # in order to identify itself to be either
				a MASTER or SLAVE for each particular test procedure (serial link test,
				DSP-to-DSP interrupt generation, and SSRAM test).
			6.	MASTER DSP for each particular test procedure generates request and sends
				specific message data to the SLAVE.	SLAVE DSP receives	request and	message
				data from the MASTER DSP, modifies message data using data modification pattern
				and returns modified message data to the MASTER DSP. MASTER DSP then verifies
				returned message data from the SLAVE and either	repeates the test with the
				next message data pattern or halts on the error condition.
			7.	Each test procedure is using its own message data array for MASTER DSP and
				message data modification patterns for the SLAVE DSP.
			8.	Each test procedure provides its own pass counter within the MASTER DSP
				environment so host DSP (if used) can track for the test procurement.
			9.	MX-SIOX RESET and TM/XIO-0/1 output pins are used as successful test
				procurement indicators at the MASTER DSP.


		DSP-to-DSP communication via serial links test procedure:
			1.	For HORIZONTAL LINKs, the following is the MASTER/SLAVE assignment:
					- DSP-A is the MASTER for communication with DSP-B (SLAVE)
					- DSP-C is the MASTER for communication with DSP-D (SLAVE)
			2.	For VERTICAL LINKs (if enabled), the following is the MASTER/SLAVE assignment:
					- DSP-A is the MASTER for communication with DSP-C (SLAVE)
					- DSP-B is the MASTER for communication with DSP-D (SLAVE)
			3.	For DIAGONAL LINKs (if enabled), the following is the MASTER/SLAVE assignment:
					- DSP-A is the MASTER for communication with DSP-D (SLAVE)
					- DSP-B is the MASTER for communication with DSP-C (SLAVE)
			4.	MASTER transmits message data over the corresponding McBSP port.
			5.	SLAVE scans for the corresponding McBSP port is ready. Receives input data,
				modifies it using dedicated rule, and returns data to the MASTER.
			6.	MASTER verifies received data from SLAVE and either proceeds with the next
				message data or terminates this test on the error condition.
			7.	MASTER provides success pass counter at predefined memory location for
				monitoring by host DSP s/w.


		DSP-to-DSP communication via direct DSP-to-DSP interrupt requests:
			1.	For HORIZONTAL IRQ test, the following is the MASTER/SLAVE assignment:
					- DSP-A is the MASTER for communication with DSP-B (SLAVE)
					- DSP-C is the MASTER for communication with DSP-D (SLAVE)
			2.	For VERTICAL IRQ test, the following is the MASTER/SLAVE assignment:
					- DSP-A is the MASTER for communication with DSP-C (SLAVE)
					- DSP-B is the MASTER for communication with DSP-D (SLAVE)
			3.	For DIAGONAL IRQ test, the following is the MASTER/SLAVE assignment:
					- DSP-A is the MASTER for communication with DSP-D (SLAVE)
					- DSP-B is the MASTER for communication with DSP-C (SLAVE)
			4.	MASTER generates corresponding IRQ to the SLAVE. If SSRAM is enabled, then
				also message data is passed to SLAVE via dedicated SSRAM memory location.
			5.	SLAVE receives IRQ from MASTER and input message data via SSRAM (if SSRAM is enabled),
				modifies it using dedicated rule, generates corresponding IRQ to the MASTER,
				and returns modified data to the MASTER via SSRAM.
			6.	MASTER recieves returned IRQ from the SLAVE, and reads returned data via SSRAM (if
				SSRAM is enabled), verifies received data from SLAVE and either proceeds with the next
				message data or terminates this test on the error condition.
			7.	MASTER provides success pass counter at predefined memory location for
				monitoring by host DSP s/w.
			8. 	The following is the map of C64xx DSP external interrupt request for
				all DSPs for this test:
					- EXT_INT6 is assigned to the HORIZONTAL IRQ
					- EXT_INT7 is assigned to the VERTICAL IRQ
					- NMI is assigned to the DIAGONAL IRQ


		DSP-to-DSP communication via SSRAM (if SSRAM is installed):
			1.	The following is the algorithm of SSRAM IRQ test:
					- DSP-A generates SSRAM IRQ to the DSP-B and passes the message data via
					  SSRAM DSP-B IRQ location
					- DSP-B, when receives SSRAM IRQ, generates SSRAM IRQ to the DSP-C,
					  decrements received message data from the DSP-A and passes it to the DSP-C
					- DSP-C, when receives SSRAM IRQ, generates SSRAM IRQ to the DSP-D,
					  decrements received message data from the DSP-B and passes it to the DSP-D
					- DSP-D, when receives SSRAM IRQ, generates SSRAM IRQ to the DSP-A,
					  decrements received message data from the DSP-C and passes it to the DSP-A
					- DSP-A, when receives SSRAM IRQ, verifies received message data, and if
					  correct, then begins new SSRAM IRQ test, otherwise terminates this test on
					  the error condition
			2.	DSP-A provides success pass counter at predefined memory location for
				monitoring by host DSP s/w.
			3. 	The following is the map of C64xx DSP external interrupt request for
				all DSPs for this test:
					- EXT_INT4 is assigned to the SSRAM IRQ


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
	- DSP bootmode: NOBOOT
	- DSP external vertical/diagonal links shall be enabled via
	  host DSP LINK CONFIGURATION register (default setting on
	  host PIOX reset condition)
*****************************************************/


// include headers
#include "stddef.h"
#include "px64xxq_dsp.h"



//======================== local variables ========================================

// MASTER test status procurement info descriptor
// -----------------------------------------------
// This descriptor is allocated to predefined memory location inside DSP on-chip memory
// in order host DSP s/w can access it at run-time and monitor test procurement.
struct MASTER_STAT_DD
{
 	// MASTER error status bits for each test
	int			horiz_link_master_err,
				vert_link_master_err,
				diag_link_master_err,
                horiz_irq_master_err,
				vert_irq_master_err,
				diag_irq_master_err,
				ssram_irq_master_err;


	// MASTER pass counters for each test
	unsigned	horiz_link_master_pass_counter,
				vert_link_master_pass_counter,
				diag_link_master_pass_counter,
				horiz_irq_master_pass_counter,
				vert_irq_master_pass_counter,
				diag_irq_master_pass_counter,
				ssram_irq_master_pass_counter;

} *MASTER_STAT_INFO;

	// allocation address for MASTER STATUS descriptor
#define	MASTER_STATUS_INFO_ADDR					0xfff00


// MASTER LINK test errors definition
// ----------------------------------
#define	TEST_NO_ERR								0
#define	TEST_TERMINATION_ERR					-1
#define	TEST_DISABLED_ERR						-2



// message data arrays and update patterns for LINK tests
// ------------------------------------------------------
unsigned D_HORIZ_LINK[] 	= { 0x12345678, 0x87654321, 0x9abcdef0, 0x0fedcba9,
								0x88887777, 0x66665555, 0x44443333, 0x22221111};

unsigned D_VERT_LINK[] 		= { 0x12349abc, 0x5678def0, 0xcba94321, 0x0fed8765,
								0x77778888, 0x55556666, 0x33334444, 0x11112222};

unsigned D_DIAG_LINK[] 		= { 0x1234def0, 0x56789abc, 0xcba98765, 0x0fed4321,
								0x77776666, 0x66668888, 0x33335555, 0x44441111};

#define	 XOR_UPDATE_PATTERN_HORIZ_LINK		0x55555555
#define	 XOR_UPDATE_PATTERN_VERT_LINK		0xaaaaaaaa
#define	 XOR_UPDATE_PATTERN_DIAG_LINK		0x5555aaaa



// message data arrays and update patterns for IRQ tests
// ------------------------------------------------------
unsigned D_HORIZ_IRQ[] 		= { 0x13579bdf, 0xfdb97531, 0x13572468, 0x86427531,
								0x88887777, 0x66665555, 0x44443333, 0x22221111};

unsigned D_VERT_IRQ[] 		= { 0x91547802, 0x5489dea6, 0x20fc19c2, 0xb1e3a057,
								0x72a69bf8, 0x5ed59260, 0x31634946, 0x1ac302e1};

unsigned D_DIAG_IRQ[] 		= { 0x12ecde31, 0x49789836, 0x91598047, 0x16ddfc4a,
								0xfd762639, 0x10568ea7, 0xd7435819, 0x3fd41851};

#define	 XOR_UPDATE_PATTERN_HORIZ_IRQ		0x55aa55aa
#define	 XOR_UPDATE_PATTERN_VERT_IRQ		0xaa55aa55
#define	 XOR_UPDATE_PATTERN_DIAG_IRQ		0xaaaa5555



// message data array for SSRAM-IRQ test
// --------------------------------------
unsigned D_SSRAM_IRQ[] 		= { 0x129386e0, 0x19636379, 0xee948e01, 0x4392fdc5,
								0x83065732, 0x61968490, 0xfac43897, 0x20248ecb};



#define	NUMBER_OF_MASTER_TEST_CCL		8



// SSRAM addresses for sending message data fm MASTER-to-SLAVE and backwards during IRQ test
// -----------------------------------------------------------------------------------------
// These addresses are used by dispatcher background task to set PTR to the message data.
#define		HORIZ_IRQ_DSPAB_SSRAM_MESSAGE_ADDR		(TPX64XXQ_SSRAM_SADDR+0x0)
#define		HORIZ_IRQ_DSPCD_SSRAM_MESSAGE_ADDR		(TPX64XXQ_SSRAM_SADDR+0x4)
#define		VERT_IRQ_DSPAC_SSRAM_MESSAGE_ADDR		(TPX64XXQ_SSRAM_SADDR+0x8)
#define		VERT_IRQ_DSPBD_SSRAM_MESSAGE_ADDR		(TPX64XXQ_SSRAM_SADDR+0xc)
#define		DIAG_IRQ_DSPAD_SSRAM_MESSAGE_ADDR		(TPX64XXQ_SSRAM_SADDR+0x10)
#define		DIAG_IRQ_DSPBC_SSRAM_MESSAGE_ADDR		(TPX64XXQ_SSRAM_SADDR+0x14)

#define		MASTER				0
#define		SLAVE				1


// synchro-var for synchronous start of demo in all DSP environments
int		demo_synchro_start_received=0;

//=================================================================================




//====================== MASTER/SLAVE functions for LINK tests ========================
// These functions are invoked from the main dispatcher loop upon the DSP node # and
// MASTER-SLAVE assignment as specified above.
//
// 	Link MASTER must synchronize its operation in two steps: sending request via
//  McBSP port and awaiting for reply to be received. This is performed via
//	dedicated status variable:
//		- 	'0' value denotes that master is ready to transmit new message data
//			and is awaitng for transmitter ready
//		- 	'1' value denotes that master has transmitted message data and is
//			now awaiting for the reply from the SLAVE
//		-	'-1' denotes that the test has been terminated on the error condition
//		-	'-2' denotes that the test cannot proceed due to the link desabled
//			(not applicable for HORIZONTAL LINK)
//
// 	Link SLAVE scans for McBSP port ready condition, and when ready, reads McBSP port data,
//	modify it, and writes back to the McBSP port transmitter. Data modification rule
//	includes XOR of input data with the predefined mask.


int			horiz_link_master_stat = 0,
			horiz_link_master_ccl_counter = 0;
unsigned	horiz_link_master_received_data;

void	horiz_link_master()
{
	switch(horiz_link_master_stat)
	{
		case 0:
			// get here when scanning for transmitter ready condition
			// (actually not required, since test loop includes data reception, so transmitter
			//  will be always ready in this state
				// check McBSP-2 XRDY. If not ready, then exit for next tim entry
			if(TPX64XXQ_MCBSP2_XRDY_IS_OFF) break;
				// write McBSP-2 data
			TPX64XXQ_SET_MCBSP2_DXR_DATA(D_HORIZ_LINK[horiz_link_master_ccl_counter]);
				// increment status flag for next entry to the receiver data wait
            horiz_link_master_stat++;
            break;


		case 1:
			// get here when scanning for receiver ready condition
				// check McBSP-2 RRDY. If not ready, then exit for next tim entry
			if(TPX64XXQ_MCBSP2_RRDY_IS_OFF) break;
				// read McBSP-2 data
			horiz_link_master_received_data=TPX64XXQ_GET_MCBSP2_DRR_DATA;
				// verify read data
            if(horiz_link_master_received_data!=(D_HORIZ_LINK[horiz_link_master_ccl_counter] ^ XOR_UPDATE_PATTERN_HORIZ_LINK))
			{
    	      	horiz_link_master_stat=-1;	// terminate cycle if verification error
        		MASTER_STAT_INFO->horiz_link_master_err = TEST_TERMINATION_ERR;
			}
            else
            {
				// successful test
				// allow next test cycle to proceed
               	horiz_link_master_stat=0;
               	horiz_link_master_ccl_counter++;
               	horiz_link_master_ccl_counter %= NUMBER_OF_MASTER_TEST_CCL;
               	MASTER_STAT_INFO->horiz_link_master_pass_counter ++;
            }
            break;


		default:
			// get here on test error termination condition
			break;
	}
}


void	horiz_link_slave()
{

	// check McBSP port RRDY condition
	// Exit of not ready.
	if(TPX64XXQ_MCBSP2_RRDY_IS_OFF) return;

	// read McBSP-2 data
	TPX64XXQ_SET_MCBSP2_DXR_DATA(TPX64XXQ_GET_MCBSP2_DRR_DATA ^ XOR_UPDATE_PATTERN_HORIZ_LINK);
}






int			vert_link_master_stat = 0,
			vert_link_master_ccl_counter = 0;
unsigned	vert_link_master_received_data;

void	vert_link_master()
{
	// check for vertical link is enabled
	if(TPX64XXQ_VERT_LINK_ENABLE_IS_OFF)
	{
        vert_link_master_stat = -2;
       	MASTER_STAT_INFO->vert_link_master_err = TEST_DISABLED_ERR;
        return;
	}


	switch(vert_link_master_stat)
	{
		case 0:
			// get here when scanning for transmitter ready condition
			// (actually not required, since test loop includes data reception, so transmitter
			//  will be always ready in this state
				// check McBSP-1 XRDY. If not ready, then exit for next tim entry
			if(TPX64XXQ_MCBSP1_XRDY_IS_OFF) break;
				// write McBSP-1 data
			TPX64XXQ_SET_MCBSP1_DXR_DATA(D_VERT_LINK[vert_link_master_ccl_counter]);
				// increment status flag for next entry to the receiver data wait
            vert_link_master_stat++;
            break;


		case 1:
			// get here when scanning for receiver ready condition
				// check McBSP-1 RRDY. If not ready, then exit for next tim entry
			if(TPX64XXQ_MCBSP1_RRDY_IS_OFF) break;
				// read McBSP-1 data
			vert_link_master_received_data=TPX64XXQ_GET_MCBSP1_DRR_DATA;
				// verify read data
            if(vert_link_master_received_data!=(D_VERT_LINK[vert_link_master_ccl_counter] ^ XOR_UPDATE_PATTERN_VERT_LINK))
			{
				// terminate cycle if verification error
				vert_link_master_stat=-1;
            	MASTER_STAT_INFO->vert_link_master_err = TEST_TERMINATION_ERR;

			}
            else
            {
				// successful test
				// allow next test cycle to proceed
               	vert_link_master_stat=0;
               	vert_link_master_ccl_counter++;
               	vert_link_master_ccl_counter %= NUMBER_OF_MASTER_TEST_CCL;
               	MASTER_STAT_INFO->vert_link_master_pass_counter ++;
            }
            break;


		default:
			// get here on test error termination condition
			break;
	}
}


void	vert_link_slave()
{
	// check for vertical link is enabled
	if(TPX64XXQ_VERT_LINK_ENABLE_IS_OFF) return;

	// check McBSP port RRDY condition
	// Exit of not ready.
	if(TPX64XXQ_MCBSP1_RRDY_IS_OFF) return;

	// read McBSP-1 data
	TPX64XXQ_SET_MCBSP1_DXR_DATA(TPX64XXQ_GET_MCBSP1_DRR_DATA ^ XOR_UPDATE_PATTERN_VERT_LINK);
}





int			diag_link_master_stat = 0,
			diag_link_master_ccl_counter = 0;
unsigned	diag_link_master_received_data;

void	diag_link_master()
{
	// check for diagonal link is enabled
	if(TPX64XXQ_DIAG_LINK_ENABLE_IS_OFF)
	{
        diag_link_master_stat = -2;
       	MASTER_STAT_INFO->diag_link_master_err = TEST_DISABLED_ERR;
        return;
	}


	switch(diag_link_master_stat)
	{
		case 0:
			// get here when scanning for transmitter ready condition
			// (actually not required, since test loop includes data reception, so transmitter
			//  will be always ready in this state
				// check McBSP-0 XRDY. If not ready, then exit for next tim entry
			if(TPX64XXQ_MCBSP0_XRDY_IS_OFF) break;
				// write McBSP-0 data
			TPX64XXQ_SET_MCBSP0_DXR_DATA(D_DIAG_LINK[diag_link_master_ccl_counter]);
				// increment status flag for next entry to the receiver data wait
            diag_link_master_stat++;
            break;


		case 1:
			// get here when scanning for receiver ready condition
				// check McBSP-0 RRDY. If not ready, then exit for next tim entry
			if(TPX64XXQ_MCBSP0_RRDY_IS_OFF) break;
				// read McBSP-0 data
			diag_link_master_received_data=TPX64XXQ_GET_MCBSP0_DRR_DATA;
				// verify read data
            if(diag_link_master_received_data!=(D_DIAG_LINK[diag_link_master_ccl_counter] ^ XOR_UPDATE_PATTERN_DIAG_LINK))
			{
				// terminate cycle if verification error
            	diag_link_master_stat=-1;
            	MASTER_STAT_INFO->diag_link_master_err = TEST_TERMINATION_ERR;

   			}
            else
            {
				// successful test
				// allow next test cycle to proceed
               	diag_link_master_stat=0;

               	diag_link_master_ccl_counter++;
               	diag_link_master_ccl_counter %= NUMBER_OF_MASTER_TEST_CCL;

               	MASTER_STAT_INFO->diag_link_master_pass_counter ++;
            }
            break;


		default:
			// get here on test error termination condition
			break;
	}
}


void	diag_link_slave()
{
	// check for vertical link is enabled
	if(TPX64XXQ_DIAG_LINK_ENABLE_IS_OFF) return;

	// check McBSP port RRDY condition
	// Exit of not ready.
	if(TPX64XXQ_MCBSP0_RRDY_IS_OFF) return;

	// read McBSP-0 data
	TPX64XXQ_SET_MCBSP0_DXR_DATA(TPX64XXQ_GET_MCBSP0_DRR_DATA ^ XOR_UPDATE_PATTERN_DIAG_LINK);
}

//=====================================================================================




//====================== MASTER functions for IRQ tests ===============================
// These functions are invoked from the main dispatcher loop in order to procure
// HORIZONTAL/VERTICAL/DIAGONAL IRQ tests  as described above.
//
// 	IRQ MASTER must synchronize its operation in two steps: sending request via
//  dedicated IRQ line and awaiting for reply IRQ to be received. This is performed via
//	dedicated status variable:
//		- 	'0' value denotes that the master is ready to generate IRQ and send new message data
//			over dedicated SSRAM location (if SSRAM is installed)
//		- 	'1' value denotes that the master has generated IRQ and sent a message data and is
//			now awaiting for the reply IRQ from the SLAVE
//		-	'2' denotes that the reply IRQ has been received and master must verify returned data.
//			This STATUS value is set by the horizontal interrupt handler.
//		-	'-1' denotes that the test has been terminated on the error condition
//
//	Dispatcher background task sets NODE TYPE vars for each IRQ test and SSRAM MESSAGE DATA PTR
//	upon the DSP node # and MASTER/SLAVE scheme as described at the top of this file.

// 	IRQ SLAVE receives IRQ, reads message data via SSRAM (if installed), modify read data, and
//	generates reply IRQ and writes data back to SSRAM. There is no dedicated SLAVE control function,
// 	IRQ is responsible for SLAVE operation.


int			horiz_irq_master_stat = 0,
			horiz_irq_master_ccl_counter = 0,
			horiz_irq_node_type = 0;			// this var identifies whether this node is MASTER or SLAVE

unsigned	horiz_irq_master_received_data,
			*horiz_irq_ssram_message_ptr;

void	horiz_irq_master()
{
	// check whether this is correct call to the MASTER
	// Exit if this is SLAVE.
 	if(horiz_irq_node_type!=MASTER) return;

	// check for the error has been set either by this MASTER function or IRQ handler
	// Note, that error value is the var, which cannot be confused during this test.
	if(MASTER_STAT_INFO->horiz_irq_master_err==TEST_TERMINATION_ERR)
	{
		// if error has been set, then set status to error and exit
        horiz_irq_master_stat = -1;
        return;
	}

	// if error is not set, then proceed with the test
	switch(horiz_irq_master_stat)
	{
		case 0:
			// get here when MASTER is ready to generate IRQ and send message data
				// check whether SSRAM is installed (PTR!=NULL)
			if(horiz_irq_ssram_message_ptr!=NULL)
			{	// write horizontal message to SSRAM address
				*horiz_irq_ssram_message_ptr = D_HORIZ_IRQ[horiz_irq_master_ccl_counter];
			}
				// increment status flag for next entry to the reply wait
            horiz_irq_master_stat++;

				// generate IRQ to SLAVE
			TPX64XXQ_set_horiz_rq();
            break;


		case 1:
			// get here when awaiting for the reply IRQ from the SLAVE. When MASTER will receive the
			// reply IRQ via interrupt handler, it will set STATUS to '2' and will copy read data to
			// the local var in order to exclude any confusions.
			// Nothing to do, just exit
			break;


		case 2:
			// get here after interrupt handler will receive reply IRQ
				// verify SSRAM read data
            if(horiz_irq_ssram_message_ptr!=NULL)
			{
				if(horiz_irq_master_received_data!=(D_HORIZ_IRQ[horiz_irq_master_ccl_counter] ^ XOR_UPDATE_PATTERN_HORIZ_IRQ))
				{
					// terminate cycle if verification error
            		horiz_irq_master_stat=-1;
            		MASTER_STAT_INFO->horiz_irq_master_err = TEST_TERMINATION_ERR;
            		// exit
            		break;
   				}
   			}

				// successful test finish
				// set control vars to allow next test cycle to proceed
           	horiz_irq_master_stat = 0;
           	horiz_irq_master_ccl_counter++;
           	horiz_irq_master_ccl_counter %= NUMBER_OF_MASTER_TEST_CCL;
           	MASTER_STAT_INFO->horiz_irq_master_pass_counter ++;
            break;


		default:
			// get here on test error termination condition
			// set error and status to the error state for sure in case status is not 0/1/2/-1 due to
			// unexpected IRQ appearance
           	horiz_irq_master_stat = -1;
			MASTER_STAT_INFO->horiz_irq_master_err  = TEST_TERMINATION_ERR;
			break;
	}
}





int			vert_irq_master_stat = 0,
			vert_irq_master_ccl_counter = 0,
			vert_irq_node_type = 0;			// this var identifies whether this node is MASTER or SLAVE

unsigned	vert_irq_master_received_data,
			*vert_irq_ssram_message_ptr;

void	vert_irq_master()
{
	// check whether this is correct call to the MASTER
	// Exit if this is SLAVE.
 	if(vert_irq_node_type!=MASTER) return;

	// check for the error has been set either by this MASTER function or IRQ handler
	// Note, that error value is the var, which cannot be confused during this test.
	if(MASTER_STAT_INFO->vert_irq_master_err==TEST_TERMINATION_ERR)
	{
		// if error has been set, then set status to error and exit
        vert_irq_master_stat = -1;
        return;
	}

	// if error is not set, then proceed with the test
	switch(vert_irq_master_stat)
	{
		case 0:
			// get here when MASTER is ready to generate IRQ and send message data
				// check whether SSRAM is installed (PTR!=NULL)
			if(vert_irq_ssram_message_ptr!=NULL)
			{	// write horizontal message to SSRAM address
				*vert_irq_ssram_message_ptr = D_VERT_IRQ[vert_irq_master_ccl_counter];
			}
				// increment status flag for next entry to the reply wait
            vert_irq_master_stat++;

				// generate IRQ to SLAVE
			TPX64XXQ_set_vert_rq();
            break;


		case 1:
			// get here when awaiting for the reply IRQ from the SLAVE. When MASTER will receive the
			// reply IRQ via interrupt handler, it will set STATUS to '2' and will copy read data to
			// the local var in order to exclude any confusions.
			// Nothing to do, just exit
			break;


		case 2:
			// get here after interrupt handler will receive reply IRQ
				// verify SSRAM read data
            if(vert_irq_ssram_message_ptr!=NULL)
			{
				if(vert_irq_master_received_data!=(D_VERT_IRQ[vert_irq_master_ccl_counter] ^ XOR_UPDATE_PATTERN_VERT_IRQ))
				{
					// terminate cycle if verification error
            		vert_irq_master_stat=-1;
            		MASTER_STAT_INFO->vert_irq_master_err = TEST_TERMINATION_ERR;
            		// exit
            		break;
   				}
   			}

				// successful test finish
				// set control vars to allow next test cycle to proceed
           	vert_irq_master_stat = 0;
           	vert_irq_master_ccl_counter++;
           	vert_irq_master_ccl_counter %= NUMBER_OF_MASTER_TEST_CCL;
           	MASTER_STAT_INFO->vert_irq_master_pass_counter ++;
            break;


		default:
			// get here on test error termination condition
			// set error and status to the error state for sure in case status is not 0/1/2/-1 due to
			// unexpected IRQ appearance
           	vert_irq_master_stat = -1;
			MASTER_STAT_INFO->vert_irq_master_err  = TEST_TERMINATION_ERR;
			break;
	}
}




int			diag_irq_master_stat = 0,
			diag_irq_master_ccl_counter = 0,
			diag_irq_node_type = 0;			// this var identifies whether this node is MASTER or SLAVE

unsigned	diag_irq_master_received_data,
			*diag_irq_ssram_message_ptr;

void	diag_irq_master()
{
	// check whether this is correct call to the MASTER
	// Exit if this is SLAVE.
 	if(diag_irq_node_type!=MASTER) return;

	// check for the error has been set either by this MASTER function or IRQ handler
	// Note, that error value is the var, which cannot be confused during this test.
	if(MASTER_STAT_INFO->diag_irq_master_err==TEST_TERMINATION_ERR)
	{
		// if error has been set, then set status to error and exit
        diag_irq_master_stat = -1;
        return;
	}

	// if error is not set, then proceed with the test
	switch(diag_irq_master_stat)
	{
		case 0:
			// get here when MASTER is ready to generate IRQ and send message data
				// check whether SSRAM is installed (PTR!=NULL)
			if(diag_irq_ssram_message_ptr!=NULL)
			{	// write horizontal message to SSRAM address
				*diag_irq_ssram_message_ptr = D_DIAG_IRQ[diag_irq_master_ccl_counter];
			}
				// increment status flag for next entry to the reply wait
            diag_irq_master_stat++;

				// generate IRQ to SLAVE
			TPX64XXQ_set_diag_rq();
            break;


		case 1:
			// get here when awaiting for the reply IRQ from the SLAVE. When MASTER will receive the
			// reply IRQ via interrupt handler, it will set STATUS to '2' and will copy read data to
			// the local var in order to exclude any confusions.
			// Nothing to do, just exit
			break;


		case 2:
			// get here after interrupt handler will receive reply IRQ
				// verify SSRAM read data
            if(diag_irq_ssram_message_ptr!=NULL)
			{
				if(diag_irq_master_received_data!=(D_DIAG_IRQ[diag_irq_master_ccl_counter] ^ XOR_UPDATE_PATTERN_DIAG_IRQ))
				{
					// terminate cycle if verification error
            		diag_irq_master_stat=-1;
            		MASTER_STAT_INFO->diag_irq_master_err = TEST_TERMINATION_ERR;
            		// exit
            		break;
   				}
   			}

				// successful test finish
				// set control vars to allow next test cycle to proceed
           	diag_irq_master_stat = 0;
           	diag_irq_master_ccl_counter++;
           	diag_irq_master_ccl_counter %= NUMBER_OF_MASTER_TEST_CCL;
           	MASTER_STAT_INFO->diag_irq_master_pass_counter ++;
            break;


		default:
			// get here on test error termination condition
			// set error and status to the error state for sure in case status is not 0/1/2/-1 due to
			// unexpected IRQ appearance
           	diag_irq_master_stat = -1;
			MASTER_STAT_INFO->diag_irq_master_err  = TEST_TERMINATION_ERR;
			break;
	}
}
//=====================================================================================





//====================== DSP-A MASTER function for SSRAM-IRQ tests ====================
// This function is invoked from the main dispatcher loop for the DSP-A in order to
// procure SSRAM IRQ test as described above. This test runs only in case SSRAM is installed.
//
// 	DSP-A MASTER, which is the only MASTER in this test, must synchronize its operation in
//	two steps: sending request to DSP-B via dedicated SSRAM location for generation IRQ to
//	DSP-B and awaiting for reply SSRAM IRQ-A to be received. This is performed via dedicated
//	MASTER status variable:
//		- 	'0' value denotes that the DSP-A is ready to send new message data and SSRAM-IRQ
//			to DSP-B via dedicated SSRAM location for generation of SSRAM-IRQ to DSP-B
//		- 	'1' value denotes that the master has sent a message data and SSRAM-IRQ to DSP-B and is
//			now awaiting for the reply SSRAM-IRQ from DSP-D
//		-	'2' denotes that the reply SSRAM-IRQ has been received and master must verify returned data.
//			This STATUS value is set by the horizontal interrupt handler.
//		-	'-1' denotes that the test has been terminated on the error condition
//		-	'-2' denotes that the test cannot be run due to SSRAM is not installed
//
//	DSP-B, DSP-C and DSP-D receive SSRAM-IRQ over dedicated SSRAM location, decrements received
//	message data and send this message and SSRAM-IRQ to DSP-C, DSP-D and DSP-A correspondingly.
//
//	MX-SIOX #A RESET output pin is used as successful test procurement indicator at DSP-A MASTER.
//	MX-SIOX RESET is set at the start of the test and is cleared at the end of the test.



int			ssram_irq_master_stat = 0,
			ssram_irq_master_ccl_counter = 0;

unsigned	ssram_irq_master_received_data;

void	ssram_irq_master()
{
	// check whether this is correct call to the DSP-A, which is the only MASTER in this test
	// Exit if this is not DSP-A.
 	if(!TPX64XXQ_MY_NODE_ID_IS_DSPA)
	  	return;

	// check whether SSRAM is installed, set error otherwise and exit
 	if(	TPX64XXQ_SSRAM_IS_NOT_INSTALLED)
 	{
        ssram_irq_master_stat = -2;
   		MASTER_STAT_INFO->ssram_irq_master_err = TEST_DISABLED_ERR;
	  	return;
	}

	// check for the error has been set either by this MASTER function or IRQ handler
	// Note, that error value is the var, which cannot be confused during this test.
	if(MASTER_STAT_INFO->ssram_irq_master_err==TEST_TERMINATION_ERR)
	{
		// if error has been set, then set status to error and exit
        ssram_irq_master_stat = -1;
        return;
	}

	// if error is not set, then proceed with the test
	switch(ssram_irq_master_stat)
	{
		case 0:
			// increment status flag for next entry to the reply wait
            ssram_irq_master_stat++;

			// get here when DSP-A MASTER is ready to generate IRQ and send message data to DSP-B via SSRAM
			// write message to the dedicated SSRAM address location for generation SSRAM IRQ to DSP-B
			TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPB(D_SSRAM_IRQ[ssram_irq_master_ccl_counter]);

			// set MXSIOX RESET output to the '0' state as test pass indicator
			TPX64XXQ_SET_SX_RESET_ON;
            break;


		case 1:
			// get here when awaiting for the reply IRQ from the DSP-D via SSRAM-IRQ for DSP-A.
			// When DSP-A MASTER will receive the reply IRQ via interrupt handler, it will set STATUS to '2'
			// and will copy read data to the local var in order to exclude any confusions.
			// Nothing to do, just exit
			break;


		case 2:
			// get here after interrupt handler for DSP-A MASTER will receive reply SSRAM-IRQ for DSP-A
				// verify SSRAM read data
			if(ssram_irq_master_received_data!=(D_SSRAM_IRQ[ssram_irq_master_ccl_counter] -3))
			{
				// terminate cycle if verification error
           		ssram_irq_master_stat=-1;
           		MASTER_STAT_INFO->ssram_irq_master_err = TEST_TERMINATION_ERR;
           		// exit
           		break;
			}

			// successful test finish
			// set control vars to allow next test cycle to proceed
           	ssram_irq_master_stat = 0;
           	ssram_irq_master_ccl_counter++;
           	ssram_irq_master_ccl_counter %= NUMBER_OF_MASTER_TEST_CCL;
           	MASTER_STAT_INFO->ssram_irq_master_pass_counter ++;

			// reset MXSIOX RESET output to the '1' state as successful test pass indicator
			TPX64XXQ_SET_SX_RESET_OFF;
            break;


		default:
			// get here on test error termination condition
			// set error and status to the error state for sure in case status is not 0/1/2/-1 due to
			// unexpected IRQ appearance
           	ssram_irq_master_stat = -1;
			MASTER_STAT_INFO->ssram_irq_master_err  = TEST_TERMINATION_ERR;
			break;
	}
}
//=====================================================================================





//=========================== Interrupt Handlers ======================================
// interrupt handler for EXT_INT4
// not used in this demo
#pragma INTERRUPT(xint4_isr)
void xint4_isr()
{
}


// interrupt handler for EXT_INT5
// corresponds to SSRAM IRQ
#pragma INTERRUPT(xint5_isr)
void xint5_isr()
{
	// now check for synchro-start var is set in order roder to exclude erroneous IRQ
	if(demo_synchro_start_received==0)	return;

	// at first, check whether SSRAM is installed
	// exit, if not installed due to this is an erroneous interrup and must never occur
	if(TPX64XXQ_SSRAM_IS_NOT_INSTALLED) return;

 	// 	now perform DSP node specific action
	switch(TPX64XXQ_get_my_node_id())
	{
		case TPX64XXQ_NODE_ID_DSPA:
			// DSP-A is the only MASTER for this test. It must receive this SSRAM-IRQ fm DSP-D only,
			// check for correct STATUS value, copy received SSRAM-IRQ@DSP-A message to the dedicated
			// message and increment STATUS

 			// check whether error value has been set
			// if error has been set, then exit immediately
			if(MASTER_STAT_INFO->ssram_irq_master_err == TEST_TERMINATION_ERR) 	return;

			// 	DSP-A MASTER must get here only when test status value is '1' in order to exclude
			//	secondary erroneous replies.
			// 	otherwise, it is treated as the error condition
			if(ssram_irq_master_stat==1)
			{
				// 	correct entry for the DSP-A MASTER
				//	read SSRAM-IRQ@DSP-A location and copy to the dedicated VAR so
				//	the background MASTER task could verify it
				ssram_irq_master_received_data = TPX64XXQ_W32_SSRAM_RQ_TO_DSPA;

				// increment STATUS VAR for synchronization with the MASTER background task
				ssram_irq_master_stat ++;
			}
			else
			{
				// this is erroneous entry, set error status and error code
				ssram_irq_master_stat = -1;
				MASTER_STAT_INFO->ssram_irq_master_err  = TEST_TERMINATION_ERR;
			}

   	    	break;


		case TPX64XXQ_NODE_ID_DSPB:
			// DSP-B gets here on request from DSP-A
			// read SSRAM-IRQ@DSP-B message, decrement it and copy to the SSRAM-IRQ@DSP-C location
			// in order to generate SSRAM IRQ to the DSP-C
			TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPC(TPX64XXQ_W32_SSRAM_RQ_TO_DSPB-1);
        	break;


		case TPX64XXQ_NODE_ID_DSPC:
			// DSP-C gets here on request from DSP-B
			// read SSRAM-IRQ@DSP-C message, decrement it and copy to the SSRAM-IRQ@DSP-D location
			// in order to generate SSRAM IRQ to the DSP-D
			TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPD(TPX64XXQ_W32_SSRAM_RQ_TO_DSPC-1);
        	break;


		default:
			// DSP-D gets here on request from DSP-C
			// read SSRAM-IRQ@DSP-D message, decrement it and copy to the SSRAM-IRQ@DSP-A location
			// in order to generate SSRAM IRQ to the DSP-A
			TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPA(TPX64XXQ_W32_SSRAM_RQ_TO_DSPD-1);
        	break;

	}
}


// interrupt handler for EXT_INT6
// corresponds to HORIZONTAL IRQ
#pragma INTERRUPT(xint6_isr)
void xint6_isr()
{
	// for DSP-B, check whether this is DEMO SYNCHRO-START from DSP-A
	if(TPX64XXQ_MY_NODE_ID_IS_DSPB && (demo_synchro_start_received==0))
	{
		// if DEMO SYNCHRO-START var is not set, then this is SYNCH IRQ from DSP-A, so set it and exit
		demo_synchro_start_received=1;
		return;
	}

	// now check for synchro-start var is set in order to exclude erroneous IRQ
	if(demo_synchro_start_received==0) return;

 	// 	checks whether I'm MASTER or SLAVE
 	if(horiz_irq_node_type==MASTER)
 	{
 		// check whether error value has been set
		// if error has been set, then exit immediately
		if(MASTER_STAT_INFO->horiz_irq_master_err == TEST_TERMINATION_ERR) 	return;

		// 	MASTER must get here only when test status value is '1' in order to exclude
		//	secondary erroneous replies.
		// 	otherwise, it is treated as the error condition
		if(horiz_irq_master_stat==1)
		{
			// 	correct entry fo the MASTER
			//	read SSRAM data from the PTR location and copy to the dedicated VAR so
			//	the background MASTER task could verify it
			if(horiz_irq_ssram_message_ptr!=NULL)
				horiz_irq_master_received_data = *horiz_irq_ssram_message_ptr;

			// increment STATUS VAR for synchronization with the MASTER background task
			horiz_irq_master_stat ++;
		}
		else
		{
			// this is erroneous entry, set error status and error code
			horiz_irq_master_stat = -1;
			MASTER_STAT_INFO->horiz_irq_master_err  = TEST_TERMINATION_ERR;
		}
	}
	else
	{
		// SLAVE must read SSRAM data at the PTR address, modify it and write back,
		// and generate reply IRQ
		if(horiz_irq_ssram_message_ptr!=NULL)
			(*horiz_irq_ssram_message_ptr) ^= XOR_UPDATE_PATTERN_HORIZ_IRQ;

		TPX64XXQ_set_horiz_rq();
	}
}


// interrupt handler for EXT_INT7
// corresponds to VERTICAL IRQ
#pragma INTERRUPT(xint7_isr)
void xint7_isr()
{
	// for DSP-C, check whether this is DEMO SYNCHRO-START from DSP-A
	if(TPX64XXQ_MY_NODE_ID_IS_DSPC && (demo_synchro_start_received==0))
	{
		// if DEMO SYNCHRO-START var is not set, then this is SYNCH IRQ from DSP-A, so set it and exit
		demo_synchro_start_received=1;
		return;
	}

	// now check for synchro-start var is set in order to exclude erroneous IRQ
	if(demo_synchro_start_received==0) return;

 	// 	checks whether I'm MASTER or SLAVE
 	if(vert_irq_node_type==MASTER)
 	{
 		// check whether error value has been set
		// if error has been set, then exit immediately
		if(MASTER_STAT_INFO->vert_irq_master_err == TEST_TERMINATION_ERR) return;

		// 	MASTER must get here only when test status value is '1' in order to exclude
		//	secondary erroneous replies.
		// 	otherwise, it is treated as the error condition
		if(vert_irq_master_stat==1)
		{
			// 	correct entry fo the MASTER
			//	read SSRAM data from the PTR location and copy to the dedicated VAR so
			//	the background MASTER task could verify it
			if(vert_irq_ssram_message_ptr!=NULL)
				vert_irq_master_received_data = *vert_irq_ssram_message_ptr;

			// increment STATUS VAR for synchronization with the MASTER background task
			vert_irq_master_stat ++;
		}
		else
		{
			// this is erroneous entry, set error status and error code
			vert_irq_master_stat = -1;
			MASTER_STAT_INFO->vert_irq_master_err  = TEST_TERMINATION_ERR;
		}
	}
	else
	{
		// SLAVE must read SSRAM data at the PTR address, modify it and write back,
		// and generate reply IRQ
		if(vert_irq_ssram_message_ptr!=NULL)
			(*vert_irq_ssram_message_ptr) ^= XOR_UPDATE_PATTERN_VERT_IRQ;

		TPX64XXQ_set_vert_rq();
	}
}



// interrupt handler for NMI
// corresponds to DIAGONAL IRQ
// NMI ISR requires special declaration convention
#pragma NMI_INTERRUPT(nmi_isr)
void nmi_isr()
{
	// for DSP-D, check whether this is DEMO SYNCHRO-START from DSP-A
	if(TPX64XXQ_MY_NODE_ID_IS_DSPD && (demo_synchro_start_received==0))
	{
		// if DEMO SYNCHRO-START var is not set, then this is SYNCH IRQ from DSP-A, so set it and exit
		demo_synchro_start_received=1;
		return;
	}

	// now check for synchro-start var is set in order roder to exclude erroneous IRQ
	if(demo_synchro_start_received==0) return;

 	// 	checks whether I'm MASTER or SLAVE
 	if(diag_irq_node_type==MASTER)
 	{
 		// check whether error value has been set
		// if error has been set, then exit immediately
		if(MASTER_STAT_INFO->diag_irq_master_err == TEST_TERMINATION_ERR) return;

		// 	MASTER must get here only when test status value is '1' in order to exclude
		//	secondary erroneous replies.
		// 	otherwise, it is treated as the error condition
		if(diag_irq_master_stat==1)
		{
			// 	correct entry fo the MASTER
			//	read SSRAM data from the PTR location and copy to the dedicated VAR so
			//	the background MASTER task could verify it
			if(diag_irq_ssram_message_ptr!=NULL)
				diag_irq_master_received_data = *diag_irq_ssram_message_ptr;

			// increment STATUS VAR for synchronization with the MASTER background task
			diag_irq_master_stat ++;
		}
		else
		{
			// this is erroneous entry, set error status and error code
			diag_irq_master_stat = -1;
			MASTER_STAT_INFO->diag_irq_master_err  = TEST_TERMINATION_ERR;
		}
	}
	else
	{
		// SLAVE must read SSRAM data at the PTR address, modify it and write back,
		// and generate reply IRQ
		if(diag_irq_ssram_message_ptr!=NULL)
			(*diag_irq_ssram_message_ptr) ^= XOR_UPDATE_PATTERN_DIAG_IRQ;

		TPX64XXQ_set_diag_rq();
	}
}

//=================================================================================



main()
{
	int i;

	// initialize DSP environment
   	TPX64XXQ_init_dsp();

	// configure MX-SIOX TM/XIO-0/1 pins as general purpose OUTPUTS in order to use them as
	// successful test procurement indicators at the MASTER DSP
	TPX64XXQ_SET_SX_XIO0_CNF_OUTPUT;
	TPX64XXQ_SET_SX_XIO1_CNF_OUTPUT;

	// assign pointer to the MASTER status info descriptor
	MASTER_STAT_INFO	= (struct MASTER_STAT_DD *) MASTER_STATUS_INFO_ADDR;

	// clear MASTER STATUS INFO descriptor
    MASTER_STAT_INFO->horiz_link_master_pass_counter 	=0;
    MASTER_STAT_INFO->vert_link_master_pass_counter 	=0;
    MASTER_STAT_INFO->diag_link_master_pass_counter 	=0;

    MASTER_STAT_INFO->horiz_link_master_err			 	=0;
    MASTER_STAT_INFO->vert_link_master_err			 	=0;
    MASTER_STAT_INFO->diag_link_master_err			 	=0;

    MASTER_STAT_INFO->horiz_irq_master_pass_counter 	=0;
    MASTER_STAT_INFO->vert_irq_master_pass_counter 		=0;
    MASTER_STAT_INFO->diag_irq_master_pass_counter 		=0;

    MASTER_STAT_INFO->horiz_irq_master_err			 	=0;
    MASTER_STAT_INFO->vert_irq_master_err			 	=0;
    MASTER_STAT_INFO->diag_irq_master_err			 	=0;

    MASTER_STAT_INFO->ssram_irq_master_pass_counter 	=0;
	MASTER_STAT_INFO->ssram_irq_master_err			 	=0;

	// clear demo synch-start var for synchronization of all DSP environments
    demo_synchro_start_received = 0;

	// 	configure node type (MASTER/SLAVE) and SSRAM message data PTR for IRQ tests
	//  upon the DSP node # and MASTER/SLAVE scheme for this test
	//	SSRAM PTR is set to the NULL value in case SSRAM is not installed.
	if(TPX64XXQ_SSRAM_IS_NOT_INSTALLED)
	{
		horiz_irq_ssram_message_ptr = NULL;
		vert_irq_ssram_message_ptr = NULL;
		diag_irq_ssram_message_ptr = NULL;
	}

	switch(TPX64XXQ_get_my_node_id())
	{
		case TPX64XXQ_NODE_ID_DSPA:
			horiz_irq_node_type = MASTER;
			vert_irq_node_type = MASTER;
			diag_irq_node_type = MASTER;

			if(TPX64XXQ_SSRAM_IS_INSTALLED)
			{
				horiz_irq_ssram_message_ptr = (unsigned *) HORIZ_IRQ_DSPAB_SSRAM_MESSAGE_ADDR;
				vert_irq_ssram_message_ptr 	= (unsigned *) VERT_IRQ_DSPAC_SSRAM_MESSAGE_ADDR;
				diag_irq_ssram_message_ptr 	= (unsigned *) DIAG_IRQ_DSPAD_SSRAM_MESSAGE_ADDR;
			}

			break;


		case TPX64XXQ_NODE_ID_DSPB:
			horiz_irq_node_type = SLAVE;
			vert_irq_node_type = MASTER;
			diag_irq_node_type = MASTER;

			if(TPX64XXQ_SSRAM_IS_INSTALLED)
			{
				horiz_irq_ssram_message_ptr = (unsigned *) HORIZ_IRQ_DSPAB_SSRAM_MESSAGE_ADDR;
				vert_irq_ssram_message_ptr 	= (unsigned *) VERT_IRQ_DSPBD_SSRAM_MESSAGE_ADDR;
				diag_irq_ssram_message_ptr 	= (unsigned *) DIAG_IRQ_DSPBC_SSRAM_MESSAGE_ADDR;
			}

			break;


		case TPX64XXQ_NODE_ID_DSPC:
			horiz_irq_node_type = MASTER;
			vert_irq_node_type = SLAVE;
			diag_irq_node_type = SLAVE;

			if(TPX64XXQ_SSRAM_IS_INSTALLED)
			{
				horiz_irq_ssram_message_ptr = (unsigned *) HORIZ_IRQ_DSPCD_SSRAM_MESSAGE_ADDR;
				vert_irq_ssram_message_ptr 	= (unsigned *) VERT_IRQ_DSPAC_SSRAM_MESSAGE_ADDR;
				diag_irq_ssram_message_ptr 	= (unsigned *) DIAG_IRQ_DSPBC_SSRAM_MESSAGE_ADDR;
			}

			break;


		default:
			// this corresponds to the DSP-D
			horiz_irq_node_type = SLAVE;
			vert_irq_node_type = SLAVE;
			diag_irq_node_type = SLAVE;

			if(TPX64XXQ_SSRAM_IS_INSTALLED)
			{
				diag_irq_ssram_message_ptr 	= (unsigned *) DIAG_IRQ_DSPAD_SSRAM_MESSAGE_ADDR;
				horiz_irq_ssram_message_ptr = (unsigned *) HORIZ_IRQ_DSPCD_SSRAM_MESSAGE_ADDR;
				vert_irq_ssram_message_ptr 	= (unsigned *) VERT_IRQ_DSPBD_SSRAM_MESSAGE_ADDR;
			}

			break;
	}


	// configure DSP EXT_INT4..7/NMI external interrupt selectors
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_NMI_SEL);				// disable NMI selector in order to enable it later
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT4_SEL);		// disabe EXT_INT4 (not used in this demo)

	// configure EXT_INT5 to SSRAM_IRQ if SSRAM is installed
	if(TPX64XXQ_SSRAM_IS_INSTALLED)
		TPX64XXQ_SET_EXT_INT_SEL_SSRAM_IRQ(TPX64XXQ_EXT_INT5_SEL);
	else
		TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT5_SEL);

	TPX64XXQ_SET_EXT_INT_SEL_HORIZ_IRQ(TPX64XXQ_EXT_INT6_SEL);	// configure EXT_INT6 to HORIZONTAL IRQ
	TPX64XXQ_SET_EXT_INT_SEL_VERT_IRQ(TPX64XXQ_EXT_INT7_SEL);	// configure EXT_INT7 to VERTICAL IRQ

	TPX64XXQ_SET_EXT_INT_SEL_DIAG_IRQ(TPX64XXQ_NMI_SEL);		// finally, configure NMI selector to DIAGONAL IRQ


	// enable/disable external DSP interrupts via DSP on-chip IER register
	C6X_INT_DISABLE(4);
   	C6X_INT_ENABLE(5);
   	C6X_INT_ENABLE(6);
   	C6X_INT_ENABLE(7);

   	C6X_INT_CLEAR_ALL;
   	C6X_GINT_ENABLE;    // set GIE bit in order to enable external DSP interrupts

	// if this is DSP-A, then wait a while and generate HORIZ/VERT/DIAG IRQs to all DSP nodes
	// in order to synchronize execution of all DSPs.
	// Note, that all DSP-A/B/C/D shall be started be started either synchronously, or DSP-A must be
	// atsrted as the last DSP node in order DSP-B/C/D could wait for the corresponding demo synchro-start
	// IRQ from the DSP-A.
    if(TPX64XXQ_MY_NODE_ID_IS_DSPA)
	{
		// wait a while to ensure that all DSP-B/C/D have started already
		for(i=0;i<100000;i++);

		// generate DSP broadcast IRQ as synchro-pulse to all HORIZ/VERT/DIAG nodes (DSP-B/C/D) in order
		// to allow DSP-B/C/D to proceed with the demo
		TPX64XXQ_set_broadcast_rq();
		demo_synchro_start_received=1;
	}
	else
	{
		// if this is not DSP-A, then await for the demo synchro-start from the DSP-A, which
		// is transferred via the corresponding DSP-to-DSP IRQ

		// loop while demo sychro-start wvar will be set
		while(demo_synchro_start_received==0);
	}


	// enter infinite test loop
	while(1)
	{
		// analyze my node ID and call corresponding MASTER/SLAVE LINK test function in accordance with
		// MASTER/SLAVE scheme for the LINK test (see test description at the top of the file)
		switch(TPX64XXQ_get_my_node_id())
		{
			case TPX64XXQ_NODE_ID_DSPA:
				// my node ID is DSP-A
				// Call MASTER procedure for all LINK tests
				horiz_link_master();
				vert_link_master();
				diag_link_master();
				break;


			case TPX64XXQ_NODE_ID_DSPB:
				// my node ID is DSP-B
				// Call MASTER procedure for vertical/diagonal LINK tests and SLAVE for horizontal test
				horiz_link_slave();
				vert_link_master();
				diag_link_master();
				break;


			case TPX64XXQ_NODE_ID_DSPC:
				// my node ID is DSP-C
				// Call MASTER procedure for horizontal LINK test and SLAVE procedure for vertical/diagonal LINK tests
				horiz_link_master();
				vert_link_slave();
				diag_link_slave();
				break;


			default:
				// my node ID is DSP-D
				// Call SLAVE procedure for all LINK tests
				horiz_link_slave();
				vert_link_slave();
				diag_link_slave();
				break;
		}

		// 	call IRQ test functions
		//  These functions proceed for MASTER nodes only and exit for SLAVE nodes.
		horiz_irq_master();
		vert_irq_master();
		diag_irq_master();


		//	call SSRAM-IRQ MASTER, which runs for DSP-A only
		ssram_irq_master();


		// set LINK and IRQ TESTs successful procuremet indicators (TM/XIO-0 and TM/XIO-1),
		// which are set for all LINK/IRQ MASTER tests running at this particular DSP node.
		// Successfull test procurement indicator is set in case all LINK and IRQ MASTER tests
		// correspondingly have no error set and pass counter is >100. Once all pass counters
		// come >100, then they are reset to the '0' state. These LINK and IRQ tests procurement
		// indicators are DSP node# specific. DSP-D is the SLAVE for all LINK and IRQ tests, so
		// it does not provide procurement indicators
		if(TPX64XXQ_MY_NODE_ID_IS_DSPA)
		{
			// DSP-A is MASTER for all LINK tests
            if(		(MASTER_STAT_INFO->horiz_link_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->vert_link_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->diag_link_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->horiz_link_master_pass_counter >=100)
				&&	(	(MASTER_STAT_INFO->vert_link_master_pass_counter >=100)
					 ||	TPX64XXQ_VERT_LINK_ENABLE_IS_OFF
					)
				&&	(	(MASTER_STAT_INFO->diag_link_master_pass_counter >=100)
					 ||	TPX64XXQ_DIAG_LINK_ENABLE_IS_OFF
					)
			  )
			{
				// clear all pass counters and toggle TM/XIO-0 output
				MASTER_STAT_INFO->horiz_link_master_pass_counter = 0;
				MASTER_STAT_INFO->vert_link_master_pass_counter = 0;
				MASTER_STAT_INFO->diag_link_master_pass_counter = 0;

				TPX64XXQ_set_sx_xio0_data(~TPX64XXQ_get_sx_xio0_data() & 0x1);
			}


			// DSP-A is MASTER for all IRQ tests
            if(		(MASTER_STAT_INFO->horiz_irq_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->vert_irq_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->diag_irq_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->horiz_irq_master_pass_counter >=100)
				&&	(MASTER_STAT_INFO->vert_irq_master_pass_counter >=100)
				&&	(MASTER_STAT_INFO->diag_irq_master_pass_counter >=100)
			  )
			{
				// clear all apss counters and toggle TM/XIO-1 output
				MASTER_STAT_INFO->horiz_irq_master_pass_counter = 0;
				MASTER_STAT_INFO->vert_irq_master_pass_counter = 0;
				MASTER_STAT_INFO->diag_irq_master_pass_counter = 0;

				TPX64XXQ_set_sx_xio1_data(~TPX64XXQ_get_sx_xio1_data() & 0x1);
			}
		}

		if(TPX64XXQ_MY_NODE_ID_IS_DSPB)
		{
			// DSP-B is MASTER for vertical/diagonal LINK tests
			if(		(TPX64XXQ_VERT_LINK_ENABLE_IS_ON || TPX64XXQ_DIAG_LINK_ENABLE_IS_ON)
				&&	(MASTER_STAT_INFO->vert_link_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->diag_link_master_err!=TEST_TERMINATION_ERR)
				&&	(	(MASTER_STAT_INFO->vert_link_master_pass_counter >=100)
					 ||	TPX64XXQ_VERT_LINK_ENABLE_IS_OFF
					)
				&&	(	(MASTER_STAT_INFO->diag_link_master_pass_counter >=100)
					 ||	TPX64XXQ_DIAG_LINK_ENABLE_IS_OFF
					)
			  )
			{
				// clear all apss counters and toggle TM/XIO-0 output
				MASTER_STAT_INFO->vert_link_master_pass_counter = 0;
				MASTER_STAT_INFO->diag_link_master_pass_counter = 0;
				TPX64XXQ_set_sx_xio0_data(~TPX64XXQ_get_sx_xio0_data() & 0x1);
			}


			// DSP-B is MASTER for vertical/diagonal IRQ tests
            if(		(MASTER_STAT_INFO->vert_irq_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->diag_irq_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->vert_irq_master_pass_counter >=100)
				&&	(MASTER_STAT_INFO->diag_irq_master_pass_counter >=100)
			  )
			{
				// clear all apss counters and toggle TM/XIO-1 output
				MASTER_STAT_INFO->vert_irq_master_pass_counter = 0;
				MASTER_STAT_INFO->diag_irq_master_pass_counter = 0;

				TPX64XXQ_set_sx_xio1_data(~TPX64XXQ_get_sx_xio1_data() & 0x1);
			}
		}


		if(TPX64XXQ_MY_NODE_ID_IS_DSPC)
		{
			// DSP-C is MASTER procedure for horizontal LINK test only
            if(		(MASTER_STAT_INFO->horiz_link_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->horiz_link_master_pass_counter >=100)
			  )
			{
				// clear all apss counters and toggle TM/XIO-0 output
				MASTER_STAT_INFO->horiz_link_master_pass_counter = 0;

				TPX64XXQ_set_sx_xio0_data(~TPX64XXQ_get_sx_xio0_data() & 0x1);
			}


			// DSP-C is MASTER for horizontal IRQ test only
            if(		(MASTER_STAT_INFO->horiz_irq_master_err!=TEST_TERMINATION_ERR)
				&&	(MASTER_STAT_INFO->horiz_irq_master_pass_counter >=100)
			  )
			{
				// clear all apss counters and toggle TM/XIO-1 output
				MASTER_STAT_INFO->horiz_irq_master_pass_counter = 0;

				TPX64XXQ_set_sx_xio1_data(~TPX64XXQ_get_sx_xio1_data() & 0x1);
			}
		}
	}
}





